;; Copyright 2009, 2010 Kevin Ryde.

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.


(defun man-completion-hook-to-list (var)
  "Return a list of functions in hook variable VAR.
VAR is a symbol, if unbound then the return is an empty list."
  (and (boundp var)
       (let ((value (symbol-value var)))
         (if (functionp value)
             (list value)
           (apply 'append
                  (mapcar (lambda (elem)
                            (if (eq t elem)
                                (let ((value (default-value var)))
                                  (if (functionp value) (list value) value))
                              (list elem)))
                          value))))))




(defun checkdoc-hook-functions (hookvar)
  "Return a list of functions HOOKVAR should run.
HOOKVAR can be a single function or list of functions, the return
is always a list.  `t' in a buffer-local value means use the
global `default-value' at that point, and it can likewise be a
function or list."
  (let ((hooks (symbol-value hookvar)))
    (if (and hooks
	     ;; same test as run_hook_with_args()
	     (or (not (consp hooks))
		 (eq 'lambda (car hooks))))
	(list hooks) ;; single function listified

      ;; expand `t' to global value
      (apply 'append
	     (mapcar (lambda (func)
		       (if (eq t func)
			   (let ((global (default-value hookvar)))
			     ;; same test as run_hook_with_args()
			     (if (or (not (consp global))
				     (eq 'lambda (car global)))
				 (list global)
			       global))
			 (list func)))
		     hooks)))))

(defun checkdoc-run-hooks-last-true (hookvar)
  "Run hooks in HOOKVAR and return the last true value."
  (let (ret)
    (dolist (func (checkdoc-hook-functions hookvar))
      (setq ret (or (funcall func) ret)))))

