;; Copyright 2009, 2010, 2013, 2017 Kevin Ryde.    -*- coding: emacs-mule -*-

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.


;;-----------------------------------------------------------------------------
(terminal-coding-system)
(unicode-disp--char-displayable-p #x2022)
(unicode-disp--char-displayable-p #x41)
(insert "\u21d0")
(encode-char ?\u2022 'iso-latin-1-unix)
(insert (encode-coding-string "\u2022" 'iso-latin-1-unix))
(encode-coding-string (char-to-string ?\u2022) 'iso-latin-1-unix)
(encode-coding-string (char-to-string ?\uABFE) 'iso-latin-1-unix)
 

;;-----------------------------------------------------------------------------
(let ((filename (buffer-file-name)))
  (require 'un-define)
  (kill-buffer nil)
  (let ((coding-system-for-read 'utf-8))
    (find-file filename)))

(defun make-glyph-code (char)
  char)
(aset standard-display-table ?X (vector (make-glyph-code ?!)))
(aset standard-display-table ?X nil)
(aref standard-display-table ?X)
(setq standard-display-table standard-display-table)
(setq standard-display-table (copy-sequence standard-display-table))
(describe-display-table standard-display-table)

(aset (window-display-table) ?X (vector (make-glyph-code ?!)))
(aset (window-display-table) ?X nil)
(describe-display-table (window-display-table))

(progn
  (add-to-list 'load-path (expand-file-name "~/lisp"))
  (require 'unicode-disp))
(unicode-disp)


(dolist (n '(#x2010
             #x2018
             #x2019
             #x201C
             #x201D
             #x2192
             #x2212
             #x2502
             #x203E
             #x2022
             #x2264
             #x2265))
  (let* ((char (decode-char 'ucs n))
         (name  (get-char-code-property char 'name))
         (ndesc (and (fboundp 'char-code-property-description)
                     (char-code-property-description 'name name))))
    (insert (format "U+%X %c %s (%S)\n"
                    (encode-char char 'ucs) char name ndesc))))
U+2010  HYPHEN (nil)
U+2018  LEFT SINGLE QUOTATION MARK (nil)
U+2019  RIGHT SINGLE QUOTATION MARK (nil)
U+201C  LEFT DOUBLE QUOTATION MARK (nil)
U+201D  RIGHT DOUBLE QUOTATION MARK (nil)
U+2192  RIGHTWARDS ARROW (nil)
U+2212  MINUS SIGN (nil)
U+2502  BOX DRAWINGS LIGHT VERTICAL (nil)
U+203E  OVERLINE (nil)
U+2022  BULLET (nil)
U+2264  LESS-THAN OR EQUAL TO (nil)
U+2265  GREATER-THAN OR EQUAL TO (nil)


(aref standard-display-table (decode-char 'ucs #x2192))
(aref standard-display-table (decode-char 'ucs #x201D))
enriched-display-table


(progn
  (split-window)
  (let ((window (selected-window)))
    (delete-window window)
    (window-buffer window)))

;;-----------------------------------------------------------------------------
;; compile

(progn
  (add-to-list 'load-path (expand-file-name "."))
  (require 'my-byte-compile)
  (my-byte-compile))

;;-----------------------------------------------------------------------------
;; xemacs maybe

(eval-and-compile
  (if (eval-when-compile (fboundp 'window-display-table))
      ;; emacs21 up
      (defalias 'formfeed-hline--window-display-table
        'window-display-table)
    ;; xemacs
    (defun formfeed-hline--window-display-table (window)
      "Return the display table for WINDOW."
      (specifier-instance current-display-table window))))

(defun formfeed-hline--buffer-display-table (buffer)
      "Return the display table for BUFFER."
  (if (eval-when-compile (boundp 'buffer-display-table))
      ;; emacs
      (with-current-buffer buffer
        buffer-display-table)
    ;; xemacs
    (specifier-instance current-display-table buffer)))


  (when (eval-when-compile (boundp 'current-display-table)) ;; xemacs
    (unicode-disp-one (specifier-instance current-display-table 'global)))


;;-----------------------------
;; unused

(defun unicode-disp-standard-parent (table)
  (and standard-display-table
       (catch 'found
         (while table
           (if (eq table standard-display-table)
               (throw 'found t))
           (setq table (char-table-parent table)))
         nil)))

(defvar unicode-disp-hashdone
  (make-hash-table :test 'eq
                   :weakness 'key))

  (unless (gethash table unicode-disp-hashdone)
    (puthash table t unicode-disp-hashdone)



    (dolist (buffer (buffer-list))
      (with-current-buffer buffer
        (let ((table buffer-display-table))
          (if table
              (unicode-disp-one table)))))


;;-----------------------------------------------------------------------------

(progn
  (defun foo (str)
    (loop for c across str
          vconcat (unicode-disp--make-glyph-code c face)))
  (byte-compile 'foo)
  (disassemble 'foo))

;;-----------------------------------------------------------------------------
;; emacs20

(progn
  (add-to-list 'load-path (expand-file-name "~/lisp"))
  (require 'unicode-disp))

  (setq debug-on-error t)
  (unicode-disp))

(formfeed-hline-mode 0)
(formfeed-hline-mode 1)
(aref standard-display-table ?\f)

(defconst unicode-disp-default-character-list
  `((,(string-to-number "2010" 16) "-");; HYPHEN
    (,(string-to-number "2018" 16) "`");; LEFT SINGLE QUOTATION MARK
    (,(string-to-number "2019" 16) "'");; RIGHT SINGLE QUOTATION MARK
    (,(string-to-number "201C" 16) "\"");; LEFT DOUBLE QUOTATION MARK
    (,(string-to-number "201D" 16) "\"");; RIGHT DOUBLE QUOTATION MARK
    (,(string-to-number "2192" 16) "->";; RIGHTWARDS ARROW
     unicode-disp-escape-face)
    (,(string-to-number "2212" 16) "-");; MINUS SIGN
    (,(string-to-number "2502" 16) "|");; BOX DRAWINGS LIGHT VERTICAL
    (,(string-to-number "203E" 16) " ";; OVERLINE as face
     unicode-disp-overline-face)))

		  

abc

