;; Copyright 2011 Kevin Ryde

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.

(add-to-list 'load-path (expand-file-name ".."))

(defun my-run-test ()

  (let ((count 0)
        (tempbuf "*t-ffap-I-option--temp*"))
    (switch-to-buffer tempbuf)

    ;;-------------------------------------------------------------------------
    ;; load

    (require 'ffap-I-option)

    ;; no leftovers in emacs21 byte compiling defadvice
    (when (and (boundp 'byte-compile-unresolved-functions)
               byte-compile-unresolved-functions)
      (error "byte-compile-unresolved-functions after load: %S"
             byte-compile-unresolved-functions))
    (setq count (1+ count))


    ;;-------------------------------------------------------------------------
    ;; ffap-string-at-point -- when never enabled

    (ad-activate 'ffap-string-at-point)

    (erase-buffer)

    (let ((got  (ffap-string-at-point))
          (want ""))
      (unless (equal got want)
        (error "ffap-string-at-point in empty buffer got: %S" got))
      (setq count (1+ count)))

    (erase-buffer)
    (insert "-I/tmp")
    (goto-char (point-min))

    (let ((got  (ffap-string-at-point))
          (want "-I/tmp"))
      (unless (equal got want)
        (error "ffap-string-at-point when never enabled got: %S" got))
      (setq count (1+ count)))


    ;;-------------------------------------------------------------------------
    ;; ffap-string-at-point -- when enabled

    (ffap-I-option-enable)

    (mapcar (lambda (elem)
              (let ((content     (nth 0 elem))
                    (pos         (nth 1 elem))
                    (want-file   (nth 2 elem))
                    (want-string (or (nth 3 elem) (nth 2 elem))))
                (erase-buffer)
                (insert content)
                (goto-char (+ (point-min) pos))

                (let ((got (ffap-string-at-point)))
                  (unless (equal want-string got)
                    (error "ffap-file-at-point got %S want %S\ncontent: %S"
                           got want-string content))
                  (unless (or (not got)
                              (equal got ffap-string-at-point))
                    (error "got %S not put into ffap-string-at-point" got)))

                (let ((got (ffap-file-at-point)))
                  (unless (equal want-file got)
                    (error "ffap-file-at-point got %S want %S\ncontent: %S"
                           got want-file content))
                  (unless (or (not got)
                              (equal got ffap-string-at-point))
                    (error "got %S not put into ffap-string-at-point" got))))

              (setq count (1+ count)))

            '(("-I/tmp" 0  "/tmp")
              ("-i/tmp" 0  nil "-i/tmp") ;; not ffap-I-option
              (" -I/tmp" 0 nil "")
              (" -I/tmp" 1   "/tmp")
              ("\n-I/tmp" 1  "/tmp")
              ("\r-I/tmp" 1  "/tmp")
              ("\t-I/tmp" 1  "/tmp")
              ("\f-I/tmp" 1  "/tmp")
              ("X-I/tmp"  1  nil "X-I/tmp")
              ("0-I/tmp"  1  nil "0-I/tmp")

              ("'-I/tmp"  1  "/tmp")
              ("'-I/tmp'" 1  "/tmp")
              ("\"-I/tmp"   1  "/tmp")
              ("\"-I/tmp\"" 1  "/tmp")

              ("'-I/tmp"  0  "/tmp")
              ("'-I/tmp'" 0  "/tmp")
              ("\"-I/tmp"   0  "/tmp")
              ("\"-I/tmp\"" 0  "/tmp")

              ("\n\n-I'/tmp"  2  "/tmp")
              ("\n\n-I'/tmp'" 4  "/tmp")
              ("\n\n-I\"/tmp"   5  "/tmp")
              ("\n\n-I\"/tmp\"" 6  "/tmp")

              ;; dubious ones ...
              ("-I-I/tmp" 0  nil "-I/tmp")
              ("-I-I/tmp" 2  nil "-I/tmp")
              ))

    ;;-------------------------------------------------------------------------
    ;; unload-feature

    (add-to-list 'load-path (expand-file-name "."))
    (require 'my-unload)

    (if (not my-unload-feature-function-p)
        (message "skip, no unload-feature function")

      (let ((feature 'ffap-I-option))
        (unless (my-advice-by-name feature)
          (error "oops, didn't find any advice"))

        (unload-feature feature t)

        (let ((lst (my-advice-by-name feature)))
          (when lst
            (error "unload-feature leftover advice: %S" lst)))
        (setq count (1+ count))))


    ;;-------------------------------------------------------------------------
    ;; unload-feature -- with advice unloaded first

    (let ((feature 'ffap-I-option))
      (if (featurep feature)
          (unload-feature feature t))
      (require feature)
      ;; if 'advice can be unloaded (no dependency expressed on it), then
      ;; see FEATURE unloads successfully too
      (when (condition-case nil
                (progn (unload-feature 'advice) t)
              (error nil))
        (unload-feature feature t)))
    (setq count (1+ count))

    ;;-------------------------------------------------------------------------
    ;; ffap-string-at-point -- still works when unloaded

    (erase-buffer)
    (let ((got  (ffap-string-at-point))
          (want ""))
      (unless (equal got want)
        (error "ffap-string-at-point in empty buffer after unload got: %S"
               got))
      (setq count (1+ count)))

    ;;-------------------------------------------------------------------------

    (bury-buffer)
    (message "  %d tests" count)
    count))
