;; Copyright 2009, 2010 Kevin Ryde

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.

(add-to-list 'load-path (expand-file-name ".."))

(defun my-run-test ()

  (let ((count 0)
        (tempbuf "*t-ffap-rfc-space--temp*"))
    (switch-to-buffer tempbuf)

    ;;-------------------------------------------------------------------------
    ;; load

    (require 'ffap-rfc-space)

    ;; no leftovers in emacs21 byte compiling defadvice
    (when (and (boundp 'byte-compile-unresolved-functions)
               byte-compile-unresolved-functions)
      (error "byte-compile-unresolved-functions after load: %S"
             byte-compile-unresolved-functions))
    (setq count (1+ count))


    ;;-------------------------------------------------------------------------
    ;; thing-at-point 'rfc
    ;; ffap-string-at-point advised

    (require 'ffap)

    (mapcar (lambda (elem)
              (let ((want (nth 0 elem))
                    (str  (nth 1 elem)))
                (if (eq t want)
                    (setq want str))
                (erase-buffer)
                (insert str)

                (goto-char (point-min))
                (while (not (eobp))
                  (let ((got (thing-at-point 'rfc)))
                    (unless (equal want got)
                      (error "thing-at-point got %S want %S" got want)))
                  (forward-char))

                (unless (string-match "\\`X" str)
                  (goto-char (point-min))
                  (while (not (eobp))
                    (let ((got (ffap-string-at-point)))
                      (unless (equal want got)
                        (error "ffap-string-at-point got %S want %S" got want))
                      (unless (equal got ffap-string-at-point)
                        (error "got %S not put into ffap-string-at-point" got)))
                    (forward-char))))

              (setq count (1+ count)))

            '((t "RFC123")
              (t "RFC 123")
              (t "RFC-123")
              (t "RFC#123")
              (nil "XRFC123")
              (nil "XRFC 123")
              (nil "XRFC-123")
              (nil "XRFC#123")
              ))

    ;;-------------------------------------------------------------------------
    ;; unload-feature

    (add-to-list 'load-path (expand-file-name "."))
    (require 'my-unload)

    (if (not my-unload-feature-function-p)
        (message "skip, no unload-feature function")

      (let ((feature 'ffap-rfc-space))
        (unless (my-advice-by-name feature)
          (error "oops, didn't find any advice"))

        (unload-feature feature t)

        (let ((lst (my-advice-by-name feature)))
          (when lst
            (error "unload-feature leftover advice: %S" lst)))
        (setq count (1+ count))))


    ;;-------------------------------------------------------------------------
    ;; unload-feature -- with advice unloaded first

    (let ((feature 'ffap-rfc-space))
      (if (featurep feature)
          (unload-feature feature t))
      (require feature)
      ;; if 'advice can be unloaded (no dependency expressed on it), then
      ;; see FEATURE unloads successfully too
      (when (condition-case nil
                (progn (unload-feature 'advice) t)
              (error nil))
        (unload-feature feature t)))
    (setq count (1+ count))

    ;;-------------------------------------------------------------------------

    (bury-buffer)
    (message "  %d tests" count)
    count))
