;; Copyright 2006, 2007, 2009, 2010, 2016 Kevin Ryde

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.

(add-to-list 'load-path (expand-file-name ".."))

(defun my-run-test ()
  (require 'miscbits-el-loaddefs)
  (let ((count 0))

    (if (not (locate-library "bindat"))
        (message "skip, bindat.el not available")

      ;;-----------------------------------------------------------------------
      ;; autoload

      (mapcar (lambda (func)
                (unless (fboundp func)
                  (error "`%S' not autoloaded" func))
                (setq count (1+ count)))
              '(imagetext-show))

      (mapcar (lambda (elem)
                (let* ((hook (nth 0 elem))
                       (func (nth 1 elem))
                       (options (get hook 'custom-options)))
                  (unless (and (listp options)
                               (memq func options))
                    (error "`%S' not an option on `%S'" func hook)))
                (setq count (1+ count)))

              '((image-mode-hook imagetext-show)))

      ;;-----------------------------------------------------------------------
      ;; load

      (require 'imagetext)

      ;; no leftovers in emacs21 byte compiling defadvice
      (when (and (boundp 'byte-compile-unresolved-functions)
                 byte-compile-unresolved-functions)
        (error "byte-compile-unresolved-functions after load: %S"
               byte-compile-unresolved-functions))
      (setq count (1+ count))


      ;;-----------------------------------------------------------------------
      ;; imagetext-save-buffer-modified-p

      (if (not (fboundp 'imagetext-save-buffer-modified-p))
          (message "skip, no imagetext-save-buffer-modified-p when byte compiled")

        (with-temp-buffer
          (imagetext-save-buffer-modified-p
           (insert "x"))
          (when (buffer-modified-p)
            (error "imagetext-save-buffer-modified-p didn't set unmodified"))
          (setq count (1+ count)))

        (with-temp-buffer
          (insert "x")
          (imagetext-save-buffer-modified-p
           (insert "y"))
          (unless (buffer-modified-p)
            (error "imagetext-save-buffer-modified-p didn't preserve modified"))
          (setq count (1+ count))))

      ;;-----------------------------------------------------------------------
      ;; unload-feature

      (add-to-list 'load-path (expand-file-name "."))
      (require 'my-unload)

      (if (not my-unload-feature-function-p)
          (message "skip, no unload-feature function")

        (let ((feature 'imagetext))
          (unless (my-advice-by-name feature)
            (error "oops, didn't find any advice"))

          (unload-feature feature t)

          (let ((lst (my-advice-by-name feature)))
            (when lst
              (error "unload-feature leftover advice: %S" lst)))
          (setq count (1+ count))))


      ;;-----------------------------------------------------------------------
      ;; unload-feature -- with advice unloaded first

      (let ((feature 'imagetext))
        (if (featurep feature)
            (unload-feature feature t))
        (require feature)
        ;; if 'advice can be unloaded (no dependency expressed on it), then
        ;; see FEATURE unloads successfully too
        (when (condition-case nil
                  (progn (unload-feature 'advice) t)
                (error nil))
          (unload-feature feature t)))
      (setq count (1+ count))

      ;;-----------------------------------------------------------------------
      (message "  %d tests" count)
      count)))
