;; Copyright 2009, 2010, 2012, 2015 Kevin Ryde

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.

(add-to-list 'load-path (expand-file-name ".."))

(defun my-run-test ()
  (require 'parasep)
  (let ((count   0)
        (tempbuf "*t-parasep--temp*"))
    (switch-to-buffer tempbuf)

    ;;-------------------------------------------------------------------------
    ;; autoloads

    (mapcar (lambda (func)
              (unless (fboundp func)
                (error "`%S' not autoloaded" func))
              (setq count (1+ count)))
            '(parasep-dashes
              parasep-empty-comments
              parasep-perl-pod
              parasep-texinfo-@*))

    ;;-------------------------------------------------------------------------
    ;; commands

    (mapcar (lambda (symbol)
              (unless (commandp symbol)
                (error "`%S' not interactive")
                (setq count (1+ count))))
            '(parasep-dashes
              parasep-empty-comments
              parasep-perl-pod
              parasep-texinfo-@*))

    ;;-------------------------------------------------------------------------
    ;; parasep-regexp-valid-p

    (require 'parasep)
    (mapcar (lambda (elem)
              (let ((want (nth 0 elem))
                    (str  (nth 1 elem)))
                (let ((got (not (not (parasep-regexp-valid-p str)))))
                  (unless (equal got want)
                    (error "parasep-regexp-valid-p %S got %S want %S"
                           str got want)))
                (setq count (1+ count))))

            '((t   "")
              (nil "\\(")
              (t   "\\(\\)")
              (t   "\\|")
              (nil "[abc")
              (t   "[abc]")))

    ;;-------------------------------------------------------------------------
    ;; parasep-regexp-split

    (require 'parasep)
    (mapcar (lambda (elem)
              (let ((str  (nth 0 elem))
                    (want (nth 1 elem)))
                (let ((got (parasep-regexp-split str)))

                  ;; incompatible change in emacs22 to keep empties -- which
                  ;; is more or less wanted for regexp checking but missing
                  ;; it doesn't hurt anything
                  (setq want (delete "" (copy-sequence want)))
                  (setq got  (delete "" (copy-sequence got)))

                  (unless (equal got want)
                    (error "parasep-regexp-split %S got %S want %S"
                           str got want)))
                (setq count (1+ count))))

            '(("" (""))
              ("[" ("["))
              ("\\)" ("\\)"))
              ("\\(\\|\\)" ("\\(\\|\\)"))
              ("a\\|" ("a" ""))
              ("a\\|\\|" ("a" "" ""))
              ("a\\|\\(x\\|y\\(\\|fdjk\\)\\)\\|z" ("a" "\\(x\\|y\\(\\|fdjk\\)\\)" "z"))))

    ;;-------------------------------------------------------------------------
    ;; parasep-add-to-regexp-var

    (require 'parasep)
    (let (got
          (case-fold-search t))
      (mapcar (lambda (elem)
                (let ((str  (nth 0 elem))
                      (add  (nth 1 elem))
                      (want (nth 2 elem))
                      ret)

                  (setq got str)
                  (setq ret (parasep-add-to-regexp-var 'got add))
                  (unless (equal ret got)
                    (error "parasep-add-to-regexp-var ret %S want %S"
                           ret got))

                  (unless (equal got want)
                    (error "parasep-add-to-regexp-var %S add %S got %S want %S"
                           str add got want)))
                (setq count (1+ count)))

              '(
                ;; doesn't work adding empty regexp in emacs21 due to
                ;; split-string
                ;; ("" "" "")

                ("[ab]" "[ab]" "[ab]")
                ("x\\|\\(a\\|b\\|c\\)\\|z" "b" "b\\|x\\|\\(a\\|b\\|c\\)\\|z")
                ("a" "b" "b\\|a")
                ("a" "A" "A\\|a"))))

    ;;-------------------------------------------------------------------------

    (mapcar (lambda (elem)
              (let ((func     (nth 0 elem))
                    (mode     (nth 1 elem))
                    (str-list (nthcdr 2 elem)))
                (unless (commandp func)
                  (error "%S not interactive" func))
                (setq count (1+ count))

                (funcall mode)
                (funcall func)

                (mapcar (lambda (str)
                          (erase-buffer)
                          (insert str)
                          (goto-char (point-min))

                          (let ((got-sep   (looking-at paragraph-separate))
                                (got-start (looking-at paragraph-start)))
                            (unless (and got-sep got-start)
                              (goto-char (point-max))
                              (insert "\n\n")
                              (insert (format "sep %S start %S\n" got-sep got-start))
                              (insert (format "sep   %S\n" paragraph-separate))
                              (insert (format "start %S\n" paragraph-start))
                              (insert "\n")
                              (insert "(looking-at paragraph-separate)\n")
                              (insert "(looking-at paragraph-start)\n")
                              (goto-char (point-min))
                              (error "no match %S %S on %S
got-sep %S got-start %S
comment-start      %S
paragraph-start    %S
paragraph-separate %S"
                                     func mode str
                                     got-sep got-start
                                     comment-start paragraph-start paragraph-separate)))
                          (setq count (1+ count)))
                        str-list)))

            '(
              ;;---------------------------------------------------------
              ;; parasep-tex-index

              (parasep-tex-index fundamental-mode
                                 "\\index{}"
                                 "\\index{Blah} "
                                 "\\index{Blah@Blah}\t"
                                 "\\index{Blah-blah@Blah\hyp{}blah}"
                                 )

              ;;---------------------------------------------------------
              ;; parasep-dashes

              (parasep-dashes fundamental-mode
                              "-------------")
              (parasep-dashes emacs-lisp-mode
                              "-------------"
                              ";;-------------"
                              ";; -------------"
                              ";;\t -------------\t \t")
              (parasep-dashes c-mode
                              "-------------"
                              "/*-------------"
                              "/* -------------"
                              "/* ------------- */"
                              "//-------------"
                              "// -------------"
                              )

              ;;---------------------------------------------------------
              ;; parasep-perl-pod

              (parasep-perl-pod perl-mode
                                "=head1 FOO"
                                "=item")

              ;;---------------------------------------------------------
              ;; parasep-perl-pod

              (parasep-perl-pod-X perl-mode
                                  "X<foo>"
                                  "X<foo> \t "
                                  "X<>"
                                  "X<foo>X<bar>"
                                  "X<foo> X<bar>"
                                  "X<foo> X<bar>\tX<quux>   ")

              ;;---------------------------------------------------------
              ;; parasep-empty-comments

              (parasep-empty-comments emacs-lisp-mode
                                      ";;"
                                      ";; "
                                      ";;\t\t")
              (parasep-empty-comments c-mode
                                      "/*"
                                      ;; only in emacs21 up
                                      ;; " * "
                                      )
              ))


    ;;-------------------------------------------------------------------------
    (bury-buffer)
    (message "  %d tests" count)
    count))
