;; Copyright 2007, 2008, 2009, 2010 Kevin Ryde

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.

(add-to-list 'load-path (expand-file-name ".."))

(defun my-run-test ()
  (require 'miscbits-el-loaddefs)
  (let ((count 0)
        (tempbuf "*t-perl-pod-coding.el temp*"))
    (switch-to-buffer tempbuf)

    ;;-------------------------------------------------------------------------
    ;; load

    (require 'perl-pod-coding)

    ;; no leftovers in emacs21 byte compiling defadvice
    (when (and (boundp 'byte-compile-unresolved-functions)
               byte-compile-unresolved-functions)
      (error "byte-compile-unresolved-functions after load: %S"
             byte-compile-unresolved-functions))
    (setq count (1+ count))


    ;;-------------------------------------------------------------------------

    (if (not (or (boundp 'auto-coding-functions)
                 (fboundp 'set-auto-coding)))
        (message "  skip, not a known coding setup style")

      (setq-default case-fold-search nil)

      (mapcar (lambda (elem)
                (let ((want (car elem))
                      (str  (cadr elem)))
                  (erase-buffer)
                  (insert str)
                  (goto-char (point-min))
                  (let ((got (perl-pod-coding-function
                              (- (point-max) (point-min)))))

                    ;; emacs21 mm-charset-to-coding-system can return `latin-1'
                    (if (eq got 'latin-1)
                        (setq got 'iso-latin-1))

                    (unless (equal got want)
                      (error "str %s\ngot %s want %s" str got want))))
                (setq count (1+ count)))

              '((iso-latin-1 "=encoding latin-1")
                (iso-latin-1 "=encoding latin-1\n")
                (iso-latin-1 "=encoding latin-1\n\n")
                (iso-latin-1 "\n=encoding latin-1\n\n")
                (nil         "    =encoding latin-1\n")
                (nil         "blah =encoding latin-1 blah\n")))
      )

    ;;-----------------------------------------------------------------------
    ;; unload-feature

    (add-to-list 'load-path (expand-file-name "."))
    (require 'my-unload)

    (let ((feature 'perl-pod-coding))
      (when (and (fboundp 'set-auto-coding) ;; not in xemacs
                 (not (boundp 'auto-coding-functions))) ;; not in emacs22 up
        ;; emacs21
        (unless (my-advice-by-name feature)
          (error "oops, didn't find any advice")))

      (unload-feature feature t)

      (when (and (fboundp 'set-auto-coding) ;; not in xemacs
                 (not (boundp 'auto-coding-functions))) ;; not in emacs22 up
        ;; emacs21
        (let ((lst (my-advice-by-name feature)))
          (when lst
            (error "unload-feature leftover advice: %S" lst))))
      (setq count (1+ count)))

    ;;-----------------------------------------------------------------------
    ;; unload-feature -- with advice unloaded first

    (let ((feature 'perl-pod-coding))
      (if (featurep feature)
          (unload-feature feature t))
      (require feature)
      ;; if 'advice can be unloaded (no dependency expressed on it), then
      ;; see FEATURE unloads successfully too
      (when (condition-case nil
                (progn (unload-feature 'advice) t)
              (error nil))
        (unload-feature feature t)))
    (setq count (1+ count))

    ;;-----------------------------------------------------------------------
    (bury-buffer)
    (message "  %d tests" count)
    count))
